import requests

from .browser import Browser


class Dogpile(Browser):

    URL = 'http://www.dogpile.com/search/web'
    SEARCH_PARAMS = {
        'q': '',      # the search term
    }

    def __init__(self, proxies=None, auth=None):
        """
        Set up a browser-imitated slurper to slurp_raw Dogpile search results.
        :type proxies: dict
        :type auth: requests.auth.HTTPProxyAuth
        """
        Browser.__init__(self, proxies, auth)

    def slurp(self, query):
        """
        Slurp Google

        :param query: The search query
        :return: HTML result, or None if there was an HTTP error
        """
        search_params = self.SEARCH_PARAMS.copy()
        search_params.update({'q': query})
        headers = self.HEADERS.copy()
        headers['User-Agent'] = self.random_user_agent()

        r = requests.get(self.URL, headers=headers, params=search_params, timeout=10.0, proxies=self.proxies, auth=self.auth)

        if not r.ok:
            print(('HTTP Error: {} for query {}'.format(r.status_code, query)))
        else:
            return r.text


__author__ = 'thorwhalen'

import requests
from serialize.khan_logger import KhanLogger
import logging


class SimpleRequest(object):

    def __init__(self, log_file_name=None, log_level=logging.INFO):
        full_log_path_and_name = KhanLogger.default_log_path_with_unique_name(log_file_name)
        self.logger = KhanLogger(file_path_and_name=full_log_path_and_name, level=log_level)

    def slurp(self, url):

        r = requests.get(url, timeout=30.0)

        if not r.ok:
            self.logger.log(level=logging.WARN, simple_request="HTTP Error: {} for url {}".format(r.status_code, url))
        else:
            self.logger.log(level=logging.INFO, simple_request="Slurped url {}".format(url))
            return r.text

if __name__ == '__main__':
    sr = SimpleRequest()


import json
import os
from nippy.importing import get_environment_variable
import random
from requests.auth import HTTPProxyAuth
import urllib.request, urllib.error, urllib.parse
import requests
import re

from selenium import webdriver
import selenium.webdriver.common.keys

import nippy.pobj


class Browser(object):
    """ Base class for slurpers """

    # Load up scraper config stuff from config file upon loading class definition
    CONFIG = json.load(open(os.path.dirname(__file__) + "/config.json"))
    USER_AGENTS = CONFIG["USER_AGENTS"]
    HEADERS = CONFIG["HEADERS"]
    try:
        ANONYMIZER_AUTH = HTTPProxyAuth(get_environment_variable('VEN_ANONYMIZER_LOGIN'),
                                        get_environment_variable('VEN_ANONYMIZER_PASS'))
        ANONYMIZER_PROXIES = {'http': get_environment_variable('VEN_ANONYMIZER_PROX_HTTP')}#,'https': get_environment_variable('VEN_ANONYMIZER_PROX_HTTPS']}
    except KeyError:
        print("VEN_ANONYMIZER_LOGIN, VEN_ANONYMIZER_PASS, and/or VEN_ANONYMIZER_PROX_HTTP missing from environment")
    try:
        PROXYMESH_AUTH = requests.auth.HTTPProxyAuth(get_environment_variable('PROXYMESH_USER'),
                                                     get_environment_variable('PROXYMESH_PASS'))
        PROXYMESH_PROXIES = {'http': 'http://us.proxymesh.com:31280'}
    except KeyError:
        print("PROXYMESH_USER and/or PROXYMESH_PASS missing from environment")


    def __init__(self, **kwargs):
        """
        Creates an instance that will use proxies and authorization if proxies & auth are provided.
        :type proxies: dict
        :type auth: requests.auth.HTTPProxyAuth
        """
        # CONFIG = json.load(open(os.path.dirname(__file__) + "/config.json"))
        default_kwargs = {
            'get_header_fun': 'header_with_random_firefox_user_agent',
            'random_agents': [],
            'header': {
                "User-Agent": "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0)",
                "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
                "Accept-Encoding": "gzip, deflate",
                "Connection": "close",
                "DNT": "1"},
            'proxies': None,
            'auth': None,
            'timeout': 10.0
        }
        self = ut.util.pobj.set_attributes(self, kwargs, default_kwargs)
        # self.random_agents = []
        # self.proxies = proxies
        # self.auth = auth
        # self.timeout = 10.0
        # self.webdriver = None

    @classmethod
    def options(cls):
        print('''
            get_header_fun:
                'header_with_random_firefox_user_agent' (default)
                'fixed_header'
        ''')

    def get_header(self):
        if self.get_header_fun == 'header_with_random_firefox_user_agent':
            return self.header_with_random_user_agent(filter_in_user_agents='^Mozilla.*')
        elif self.get_header_fun == 'fixed_header':
            return self.header
        else:
            raise ValueError("Unknown get_header_fun value")

    def header_with_random_user_agent(self, filter_in_user_agents=None):
        """
        Returns a header with a random user agent
        """
        headers = self.HEADERS.copy()
        headers['User-Agent'] = self.random_user_agent(filter_in_user_agents=filter_in_user_agents)
        return headers

    def random_user_agent(self, filter_in_user_agents=None):
        """
        Returns a random user agent from the full list of user agents.
        Cycles through all agents before re-sampling from the full list again.
        """
        if not self.random_agents:
            self.random_agents = self.USER_AGENTS[:]
            if filter_in_user_agents:
                self.random_agents = list(filter(re.compile(filter_in_user_agents).search, self.random_agents))
        random.shuffle(self.random_agents)
        return self.random_agents.pop()

    def get_html_through_tor_unfinished(self, url):
        UserWarning("This hasn't really be coded yet...")
        proxy_support = urllib.request.ProxyHandler({"http": "127.0.0.1:8118"})
        opener = urllib.request.build_opener(proxy_support)
        opener.addheaders = [('User-agent', self.random_user_agent())]
        return opener.open(url).read()

    def get_html_through_requests(self, url, url_params={}, timeout=None):
        r = self.get_response_through_requests(url=url, url_params=url_params, timeout=timeout)
        # return the text if no error
        if not r.ok:
            raise ValueError('HTTP Error: {} for url {} (headers="{}"'.format(r.status_code, url, str(r.headers)))
        else:
            return r.text

    def get_response_through_requests(self, url, url_params={}, timeout=None):
        timeout = timeout or self.timeout
        header = self.header_with_random_user_agent()
        # get the content for the url
        r = requests.get(url, headers=header, params=url_params,
                         timeout=timeout, proxies=self.proxies, auth=self.auth)
        return r

    def get_html_through_selenium(self, url, url_params={}, timeout=None):
        ValueError("You haven't written this yet!!")
        pass
        # timeout = timeout or self.timeout
        # header = self.header_with_random_user_agent()
        # # get the content for the url
        # r = requests.get(url, headers=header, params=url_params,
        #                  timeout=timeout, proxies=self.proxies, auth=self.auth)
        # # return the text if no error
        # if not r.ok:
        #     raise ValueError('HTTP Error: {} for url {} (user-agent="{}"'.format(r.status_code, url, header['User-Agent'] ))
        # else:
        #     return r.text

    @classmethod
    def with_ven_anonymizer(cls):
        return Browser(
            proxies={'http': get_environment_variable('VEN_ANONYMIZER_PROX_HTTP')},
            auth=HTTPProxyAuth(get_environment_variable('VEN_ANONYMIZER_LOGIN'),
                               get_environment_variable('VEN_ANONYMIZER_PASS')))

    @classmethod
    def with_proxymesh(cls):
        return Browser(
            proxies=cls.PROXYMESH_PROXIES,
            auth=cls.PROXYMESH_AUTH)

    @classmethod
    def firefox_selenium(cls, **kwargs):
        default_kwargs = {
            'webdriver': webdriver.Firefox()
        }
        kwargs = dict(default_kwargs, **kwargs)
        return Browser(**kwargs)


class Selenium(Browser):
    def __init__(self, **kwargs):
        default_kwargs = {
            'header': {
                "User-Agent": "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0)",
                "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
                "Accept-Encoding": "gzip, deflate",
                "Connection": "close",
                "DNT": "1"},
            'random_agents': [],
            'proxies': None,
            'auth': None,
            'timeout': 10.0,
            'webdriver': 'firefox'
        }
        self = ut.util.pobj.set_attributes(self, dict(default_kwargs, **kwargs), Browser().__dict__)
        if self.webdriver == 'firefox':
            self.webdriver = webdriver.Firefox()
        elif self.webdriver == 'phantomjs':
            self.webdriver = webdriver.PhantomJS()

    def goto_url(self, url):
        self.webdriver.get(url)

    def get_html(self, url=None):
        if url:
            self.goto_url(url)
        return self.webdriver.page_source




__author__ = 'thor'

from bs4 import BeautifulSoup
from functools import wraps


@wraps(BeautifulSoup.find_all, assigned=('__module__', '__qualname__', '__annotations__', '__name__'))
def gen_find(tag, *args, **kwargs):
    """Does what BeautifulSoup.find_all does, but as an iterator.
        See find_all documentation for more information."""
    if isinstance(tag, str):
        tag = BeautifulSoup(tag)
    next_tag = tag.find(*args, **kwargs)
    while next_tag is not None:
        yield next_tag
        next_tag = next_tag.find_next(*args, **kwargs)


from mappyng.to import word_replacer

url_encode_yboss_map = {' ': '%20',
                        '"': '%22',
                        '#': '%23',
                        '$': '%24',
                        '%': '%25',
                        '&': '%26',
                        '(': '%28',
                        ')': '%29',
                        '*': '%2A',
                        '+': '%2B',
                        ',': '%2C',
                        '/': '%2F',
                        ':': '%3A',
                        ';': '%3B',
                        '<': '%3C',
                        '=': '%3D',
                        '>': '%3E',
                        '?': '%3F',
                        '@': '%40',
                        '[': '%5B',
                        '\\': '%5C',
                        ']': '%5D',
                        '^': '%5E',
                        '`': '%60',
                        '{': '%7B',
                        '|': '%7C',
                        '}': '%7D'}

url_encode_yboss = word_replacer(url_encode_yboss_map, inter_token_re='')

# def mk_url_encode_yboss_map():
#     import requests
#     import re
#     from bs4 import BeautifulSoup
#     r = requests.get('http://developer.yahoo.com/boss/search/boss_api_guide/reserve_chars_esc_val.html')
#     b = BeautifulSoup(r.text)
#     t = b.find('div', attrs={'class':'informaltable'}).find('tbody')
#     tt = t.findAll('tr')[1:]
#
#     w = [x.findAll('td') for x in tt]
#     mapfrom = [x[0] for x in w]
#     mapto = [x[1] for x in w]
#
#     mapfrom = [x.text for x in mapfrom]
#     mapfrom = [re.findall('(?<=\().(?=\))',x)[0] for x in mapfrom]
#     mapto = [x.text for x in mapto]
#     mapto = [x.replace('\n','') for x in mapto]
#
#     map_rtf_3986 = {f:t for f,t in zip(mapfrom,mapto)}
#     return map_rtf_3986



__author__ = 'thor'

import os
from nippy.importing import get_environment_variable
import oauth2
import time
import httplib2
import json
import pandas as pd

import ut as ms
from vampyre.util import url_encode_yboss
from mappyng.get import mk_fixed_coordinates_value_getter
from scrapyng.yboss import YbossText



yboss_root_url = "http://yboss.yahooapis.com"
yboss_search_root_url = yboss_root_url + '/ysearch/'

default_universal_args = {
    'start': 0,
    'count': 50,
    'market': 'en-us',
    'format': 'json'
}

service_default_req_args = {
    'web': {'count': 50, 'abstract': 'long', 'style': 'raw'},  # filter, type, view, title, url, sites
    'limitedweb': {'count': 50, 'abstract': 'long', 'style': 'raw'},  # filter, type, view, title, url, sites
    'news': {'count': 50, 'style': 'raw'},  # age, sort, title, url
    'blogs': {'count': 20, 'style': 'raw'},  # age, sort, count, title, url
    'related': {'count': 10},  # age, sort, count, title, url
    'images': {'count': 35}  # filter, queryfilter, dimensions, referurl, url
}

default_yboss_attrs = {
    'oauth_consumer_key': get_environment_variable('MON_YB_KEY'),
    'oauth_consumer_secret': get_environment_variable('MON_YB_SECRET'),
    'default_service': 'limitedweb',
    'default_params': {},
    'default_save_folder': os.getcwd()
}

service_list = ['limitedweb', 'web', 'blogs', 'news', 'related', 'images']

major_cols = ['query', 'position', 'title', 'abstract', 'dispurl', 'num_of_slurped_results', 'author']
minor_cols = ['date', 'url', 'clickurl']


class Yboss(object):

    default_yboss_attrs = {
        'oauth_consumer_key': get_environment_variable('MON_YB_KEY'),
        'oauth_consumer_secret': get_environment_variable('MON_YB_SECRET'),
        'default_service': 'limitedweb',
        'default_params': {},
        'default_save_folder': os.getcwd()
    }

    def __init__(self, **kwargs):
        self.__dict__.update(Yboss.default_yboss_attrs)
        self.__dict__.update(kwargs)
        self.consumer = oauth2.Consumer(key=self.oauth_consumer_key, secret=self.oauth_consumer_secret)

    ####################################################################################
    ###### SLURPERS ####################################################################
    def slurp_raw(self, query, service=None, params=None):
        service, params = self.fill_with_defaults(service, params)
        url = self.url(query, service, params)

        request_params = {
            'oauth_version': '1.0',
            'oauth_nonce': oauth2.generate_nonce(),
            'oauth_timestamp': int(time.time()),
        }
        oauth_request = oauth2.Request(method='GET', url=url, parameters=request_params)
        oauth_request.sign_request(oauth2.SignatureMethod_HMAC_SHA1(), self.consumer, None)
        oauth_header = oauth_request.to_header(realm='yahooapis.com')

        # Get search results
        http = httplib2.Http()
        resp, content = http.request(url, 'GET', headers=oauth_header)
        return {'content': content, 'resp': resp}
        # return "{'resp': %s, 'content': %s}" % (resp, content)

    def slurp_content(self, query, service=None, params=None):
        resp_content = self.slurp_raw(query, service=service, params=params)
        return resp_content['content']

    def slurp_content_as_dict(self, query, service=None, params=None):
        return json.loads(self.slurp_content(query, service=service, params=params))

    def slurp_content_and_save(self, query, service=None, params=None, filepath=None):
        filepath = self.get_filepath_for_params(query=query, service=service, params=params, filepath=filepath)
        resp_content = self.slurp_raw(query, service=service, params=params)
        json.dump(resp_content['content'], open(filepath, 'w'))

    def slurp_df_and_save(self, query, service=None, params=None, filepath=None, n_pages=1):
        filepath = self.get_filepath_for_params(query=query, service=service, params=params, filepath=filepath)
        df = self.slurp_results_df_multiple_pages(query=query, service=service, params=params, n_pages=n_pages)
        pd.to_pickle(df, filepath)
        return df

    def get_df(self, query, service=None, params=None, filepath=None, n_pages=1, overwrite=False):
        filepath = self.get_filepath_for_params(query=query, service=service, params=params, filepath=filepath)
        if not overwrite and os.path.exists(filepath):
            return pd.read_pickle(filepath)
        else:
            return self.slurp_df_and_save(query=query, service=service, params=params, n_pages=n_pages)

    def slurp_results_df(self, query, service=None, params=None):
        content_dict = json.loads(self.slurp_content(query, service=service, params=params))
        content_dict = self.get_item(content_dict)
        return self.content_to_results_df(content_dict)

    def content_to_results_df(self, content_dict):
        df = pd.DataFrame(content_dict['results'])
        start_position = int(content_dict['start'])
        df['position'] = list(range(start_position, start_position+len(df)))
        df._metadata = {'totalresults': int(content_dict['totalresults'])}
        return df

    def slurp_results_df_multiple_pages(self, query, service=None, params=None, n_pages=5):
        service, params = self.fill_with_defaults(service, params)
        df = pd.DataFrame()
        new_df = pd.DataFrame()
        for i in range(n_pages):
            # print "slurping %d/%d" % (i, n_pages-1)
            try:
                new_df = self.slurp_results_df(query, service=service, params=params)
                df = pd.concat([df, new_df])
            except:
                break
            params['start'] += params['count']
        df._metadata = new_df._metadata
        return df


    ####################################################################################
    ###### UTILS #######################################################################


    ####################################################################################
    ###### CONTENT ACCESSORS ###########################################################

    def load_json_dict(self, filepath):
        filepath = self.get_filepath(filepath)
        return json.loads(json.load(open(filepath, 'r')))

    def get_service_results_getter(self, service):
        service = service or self.default_save_folder
        return mk_fixed_coordinates_value_getter(['bossresponse', service])

    @classmethod
    def get_results(cls, content_dict):
        return Yboss.get_item(content_dict)['results']

    # def get_results

    @classmethod
    def get_totalresults(cls, content_dict):
        return int(Yboss.get_item(content_dict)['totalresults'])

    @classmethod
    def get_item(cls, content_dict):
        content_dict = content_dict['bossresponse']
        return content_dict[list(content_dict.keys())[0]]

    @classmethod
    def mk_fixed_coordinates_value_getter(cls, coord_list):
        return mk_fixed_coordinates_value_getter(['bossresponse'] + coord_list)

    ####################################################################################
    ###### UTILS #######################################################################
    def url(self, query, service=None, params=None):
        service = service or self.default_service
        params = params or self.default_params
        return yboss_search_root_url + self.rel_url(query=query, service=service, params=params)

    def rel_url(self, query, service=None, params=None):
        service = service or self.default_service
        params = params or self.default_params
        params = Yboss.mk_req_params(service, params)
        return "%s?q=%s%s" % (service, self.url_encode_str(query), self.url_encode_params(params))

    def get_filename_for_query(self, query, service=None, params=None):
        return self.rel_url(query, service=service, params=params).replace('?', '--')

    def get_filepath(self, filespec):
        if os.path.exists(filespec):
            file_path = filespec
        else:
            file_path = os.path.join(self.default_save_folder, filespec)
            if not os.path.exists(file_path):
                # assume it's a query, and derive what the filepath should be
                file_path = os.path.join(self.default_save_folder, self.get_filename_for_query(filespec))
        return file_path

    def get_filepath_for_params(self, query, service=None, params=None, filepath=None):
        filepath = filepath or self.default_save_folder
        if os.path.isdir(filepath):  # if filepath is a directory, need to make a filename for it
            filepath = os.path.join(filepath, self.get_filename_for_query(query, service=service, params=params))
        return filepath

    def fill_with_defaults(self, service=None, params=None):
        service = service or self.default_service
        params = Yboss.mk_req_params(service, params)
        return service, params

    @classmethod
    def mk_req_params(cls, service, params=None):
        params = params or {}
        return dict(
            dict(default_universal_args, **service_default_req_args[service]),
            **params)

    @classmethod
    def url_encode_str(cls, s):
        return url_encode_yboss(s)

    @classmethod
    def url_encode_params(cls, params):
        u = ''
        for p, v in params.items():
            if isinstance(v, str):
                u += '&%s=%s' % (p, v)
            else:
                u += '&%s=%s' % (p, str(v))
        return u

    @classmethod
    def print_some_resources(cls):
        print('''
            guide to yahoo BOSS: http://developer.yahoo.com/boss/search/boss_api_guide/index.html
            pricing (by service): http://developer.yahoo.com/boss/search/#pricing
            services: web, limitedweb, images, news, blogs, related
            response fields: http://developer.yahoo.com/boss/search/boss_api_guide/webv2_response.html
            market and languages: http://developer.yahoo.com/boss/search/boss_api_guide/supp_regions_lang.html
                ''')

    @classmethod
    def process_df(cls, df):
        df['dispurl'] = df['dispurl'].map(YbossText.remove_html_bold)
        df['title'] = df['title'].apply(YbossText.html2text)
        df['abstract'] = df['abstract'].apply(YbossText.html2text)
        df = ms.daf.manip.reorder_columns_as(df, major_cols + minor_cols)
        df = df.reset_index(drop=True)
        return df





import requests

from vampyre.browser import Browser


class Google(Browser):

    URL = 'http://www.google.com/search'
    SEARCH_PARAMS = {
        'q': '',      # the search term
        'num': '10',  # the number of results per page this
        'pws': '0'    # personalization turned off
    }

    def __init__(self, proxies=None, auth=None, num_results=10):
        """
        Set up a browser-imitated slurper to slurp_raw Google search results.
        :type proxies: dict
        :type auth: requests.auth.HTTPProxyAuth
        :type num_results: into
        """
        Browser.__init__(self, proxies, auth)
        self.num_results = num_results

    @classmethod
    def with_anon(cls, num_results=10):
        return cls(cls.ANONYMIZER_PROXIES, cls.ANONYMIZER_AUTH, num_results)

    @classmethod
    def with_not_anon(cls, num_results=10):
        return cls(num_results=num_results)

    def safe_slurp(self, query):
        """
        This method is a replacement for the method 'slurp_raw'. The old method is deprecated and should be removed
        when it is verified that it is not used in a any notebooks.
        This method is 'safer' in that it raises an exception if the response is not OK.

        Slurp Google

        :param query: The search query
        :return: HTML result, or None if there was an HTTP error
        """
        search_params = self.SEARCH_PARAMS.copy()
        search_params.update({'q': query, 'num': self.num_results})
        headers = self.HEADERS.copy()
        headers['User-Agent'] = self.random_user_agent()

        r = requests.get(self.URL, headers=headers, params=search_params,
                         timeout=10.0, proxies=self.proxies, auth=self.auth)

        if not r.ok:
            raise ValueError('HTTP Error: {} for query {}'.format(r.status_code, query))
        else:
            return r.text

    def slurp(self, query):
        """
        !!! Deprecated !!!
        Use safe_slurp() instead

        Slurp Google

        :param query: The search query
        :return: HTML result, or None if there was an HTTP error
        """
        search_params = self.SEARCH_PARAMS.copy()
        search_params.update({'q': query, 'num': self.num_results})
        headers = self.HEADERS.copy()
        headers['User-Agent'] = self.random_user_agent()

        r = requests.get(self.URL, headers=headers, params=search_params,
                         timeout=10.0, proxies=self.proxies, auth=self.auth)

        if not r.ok:
            print(('HTTP Error: {} for query {}'.format(r.status_code, query)))
        else:
            return r.text


__author__ = 'thorwhalen'


import urllib.request, urllib.error, urllib.parse
from collections import OrderedDict
import pandas as pd

VENERE_URL = "www.venere.com"
HTTP_PREFIX = "http://"
HTTP_VENERE_ROOT_URL = "http://www.venere.com/"

from bs4 import BeautifulSoup

def get_destination_page_info(geoid):
    response = urllib.request.urlopen(geoid_to_destination_url(geoid))
    finalurl = response.geturl()
    finalurl = finalurl.replace(VENERE_URL,'')
    finalurl = finalurl.replace(HTTP_PREFIX,'')
    html = response.read()
    s = BeautifulSoup(html)
    t = s.find(name='div', attrs={'id':'matchinghotels'})
    tt = t.find(name='p', attrs={'id':'results'})
    num_of_hotels = int(tt.getText())
    response.close()
    return {'geo_id':geoid, 'num_of_hotels':num_of_hotels, 'url_stub':finalurl}

def geoid_to_destination_url(geoid):
    return HTTP_VENERE_ROOT_URL + 'search/?geoid=%d' % geoid
